import { test, expect } from '@playwright/test'
import { loginAsAdmin, loginAsUser, gotoViewEnabledCredential, expectedCredentialSubjectForUser, testImportDataFile_HappyPath, testImportDataFile_SadPath } from '../src/steps';
import { ViewCredentialPage } from '../src/page-objects/US_ViewCredentialPage.js'
import { URL_IDHUB, USER1_EMAIL, USER2_EMAIL } from '../src/constants/env_constants';
import { ALERT_FILE_TO_IMPORT_IS_EMPTY, ALERT_FILE_TO_IMPORT_WITHOUT_REQUIRED_COLUMS_CC, ALERT_FILE_TO_IMPORT_WITH_ALIEN_COLUMS, ALERT_FILE_TO_IMPORT_WITH_REQUIRED_COLUMS_EMPTY, FILE_TO_IMPORT_CC, FILE_TO_IMPORT_CC_EMPTY, FILE_TO_IMPORT_CC_WITHOUT_REQUIRED_COLUMNS, FILE_TO_IMPORT_CC_WITH_ALIEN_COLUMNS, FILE_TO_IMPORT_CC_WITH_REQUIRED_EMPTY, JSON_SCHEMA_CC, SCHEMA_CC, SCHEMA_TYPE_CC } from '../src/constants/constants';

/**
* Checking data section: view the lists of files imported, import data, delete...
*/

test.describe('ADMIN-> DATA Section Tests - Testing with Course Credential', () => {

    test.beforeEach(async ({ page }) => {
        await loginAsAdmin(page, URL_IDHUB);
    })
    test.afterEach(async ({ page }) => {
        await page.click('.logout');
        await page.close();
    })

    /**
   * Load of an excel file - Happy Path:
   * Expected behavior: the file is loaded, the message:"The file was imported successfully!" is displayed,
   * and the file appears in the imported files view. 
   */

    test('DATA -> Import data file - Happy path - Course Credential - course-credential.xlsx ', async ({ page }) => {

        const fileToImport = FILE_TO_IMPORT_CC;
        const jsonSchema = JSON_SCHEMA_CC;
        const schema = SCHEMA_CC;

        await testImportDataFile_HappyPath(page, fileToImport, jsonSchema, schema);

    });

    /**
    * Load of an excel file - Sad Path:
    * Try to load a well-formatted excel file but without data.
    * Expected behavior: The error message: "The file you try to import is empty"
    */

    test('DATA -> Import data file - Sad path (file well formatted but empty) - Course Credential ', async ({ page }) => {

        const fileToImport = FILE_TO_IMPORT_CC_EMPTY;
        const jsonSchema = JSON_SCHEMA_CC;
        const schema = SCHEMA_CC;
        const unsuccessAlertMessage = ALERT_FILE_TO_IMPORT_IS_EMPTY;

        await testImportDataFile_SadPath(page, fileToImport, jsonSchema, schema, unsuccessAlertMessage);

    });

    test('DATA -> Import data file - Sad path (bad formatted file, without required columns) - Course Credential ', async ({ page }) => {

        const fileToImport = FILE_TO_IMPORT_CC_WITHOUT_REQUIRED_COLUMNS;
        const jsonSchema = JSON_SCHEMA_CC;
        const schema = SCHEMA_CC;
        const unsuccessAlertMessage = ALERT_FILE_TO_IMPORT_WITHOUT_REQUIRED_COLUMS_CC;

        await testImportDataFile_SadPath(page, fileToImport, jsonSchema, schema, unsuccessAlertMessage);

    });

    test('DATA -> Import data file - Sad path (bad formatted file, with alien columns) - Course Credential ', async ({ page }) => {


        const fileToImport = FILE_TO_IMPORT_CC_WITH_ALIEN_COLUMNS;
        const jsonSchema = JSON_SCHEMA_CC;
        const schema = SCHEMA_CC;
        const unsuccessAlertMessage = ALERT_FILE_TO_IMPORT_WITH_ALIEN_COLUMS;

        await testImportDataFile_SadPath(page, fileToImport, jsonSchema, schema, unsuccessAlertMessage);


    });

    test('DATA -> Import data file - Sad path (file with required columns present but empty) - Course Credential ', async ({ page }) => {

        const fileToImport = FILE_TO_IMPORT_CC_WITH_REQUIRED_EMPTY;
        const jsonSchema = JSON_SCHEMA_CC;
        const schema = SCHEMA_CC;
        const unsuccessAlertMessage = ALERT_FILE_TO_IMPORT_WITH_REQUIRED_COLUMS_EMPTY;

        await testImportDataFile_SadPath(page, fileToImport, jsonSchema, schema, unsuccessAlertMessage);

    });

})

test.describe('USER -> Credentials Section Tests - testing with USER1_EMAIL AND Course Credential', () => {

    test.afterEach(async ({ page }) => {
        await page.click('.logout');
        await page.close();
    })

    /**
     * PRE-CONDITIONS: the admin  has enabled a credential of type 'Course' for USER1_EMAIL
     * This is true, if the before test (DATA -> Import data- HAPPY PATH has been passed sucessfully)
    * SUMMARY:
    * - Check if the user1 can visualize the credentials that has been enabled in "My Credentials"
    * - Check the fields displayed when user click "View" Credential
    */

    test('USER Credentials -> My Credentials -> View enabled Course Credential - for user1@example.org', async ({ page }) => {
        // View the Course Credential in status 'Enabled' for the user
        await loginAsUser(page, USER1_EMAIL, URL_IDHUB);
        await gotoViewEnabledCredential(page, SCHEMA_TYPE_CC);
        const enabledCredentialView = new ViewCredentialPage(page);
        //Check that required fields exist and have a valid value in the current enabled credential

        //Get the credential subject values of the  credential visualized in the screen and compare to the model 
        let actualCredential = await enabledCredentialView.buildACredentialFromInputValues(SCHEMA_TYPE_CC);
        let expectedCredential = await expectedCredentialSubjectForUser(USER1_EMAIL, SCHEMA_TYPE_CC);
        expect(actualCredential).toEqual(expectedCredential);
    });

    test('USER Credentials -> My Credentials -> View enabled Course Credential - for user2@example.org', async ({ page }) => {
        // View the Course Credential in status 'Enabled' for the user
        await loginAsUser(page, USER2_EMAIL, URL_IDHUB);
        await gotoViewEnabledCredential(page, SCHEMA_TYPE_CC);
        const enabledCredentialView = new ViewCredentialPage(page);
        //Check that required fields exist and have a valid value in the current enabled credential

        //Get the credential subject values of the  credential visualized in the screen and compare to the model 
        let actualCredential = await enabledCredentialView.buildACredentialFromInputValues(SCHEMA_TYPE_CC);
        let expectedCredential = await expectedCredentialSubjectForUser(USER2_EMAIL, SCHEMA_TYPE_CC);
        expect(actualCredential).toEqual(expectedCredential);
    });

})

